@@ -1,27 +1,50 @@
 import inspect
+import functools
 import re
 import traceback
 import typing as ty
 import warnings
 
-import click
+try:
+    import asyncclick as click
+except ImportError:
+    import click
+import click.core
 from docutils import nodes
-from docutils import statemachine
-from docutils.nodes import section
 from docutils.parsers import rst
 from docutils.parsers.rst import directives
+from docutils import statemachine
+from sphinx import application
 from sphinx.util import logging
 from sphinx.util import nodes as sphinx_nodes
-from sphinx import application
+from sphinx.ext.autodoc import mock
 
 LOG = logging.getLogger(__name__)
 
 NESTED_FULL = 'full'
 NESTED_SHORT = 'short'
 NESTED_NONE = 'none'
+NestedT = ty.Literal['full', 'short', 'none', None]
 
 ANSI_ESC_SEQ_RE = re.compile(r'\x1B\[\d+(;\d+){0,2}m', flags=re.MULTILINE)
 
+_T_Formatter = ty.Callable[[click.Context], ty.Generator[str, None, None]]
+
+
+def _process_lines(event_name: str) -> ty.Callable[[_T_Formatter], _T_Formatter]:
+    def decorator(func: _T_Formatter) -> _T_Formatter:
+        @functools.wraps(func)
+        def process_lines(ctx: click.Context) -> ty.Generator[str, None, None]:
+            lines = list(func(ctx))
+            if "sphinx-click-env" in ctx.meta:
+                ctx.meta["sphinx-click-env"].app.events.emit(event_name, ctx, lines)
+            for line in lines:
+                yield line
+
+        return process_lines
+
+    return decorator
+
 
 def _indent(text: str, level: int = 1) -> str:
     prefix = ' ' * (4 * level)
@@ -41,7 +64,7 @@ def _get_usage(ctx: click.Context) -> str:
     return formatter.getvalue().rstrip('\n')  # type: ignore
 
 
-def _get_help_record(opt: click.Option) -> ty.Tuple[str, str]:
+def _get_help_record(ctx: click.Context, opt: click.core.Option) -> ty.Tuple[str, str]:
     """Re-implementation of click.Opt.get_help_record.
 
     The variant of 'get_help_record' found in Click makes uses of slashes to
@@ -77,18 +100,23 @@ def _get_help_record(opt: click.Option) -> ty.Tuple[str, str]:
 
     extras = []
 
-    if isinstance(opt.show_default, str):
+    if opt.show_default is not None:
+        show_default = opt.show_default
+    else:
+        show_default = ctx.show_default
+
+    if isinstance(show_default, str):
         # Starting from Click 7.0 show_default can be a string. This is
         # mostly useful when the default is not a constant and
         # documentation thus needs a manually written string.
-        extras.append(':default: %s' % opt.show_default)
-    elif opt.default is not None and opt.show_default:
+        extras.append(':default: ``%r``' % ANSI_ESC_SEQ_RE.sub('', show_default))
+    elif show_default and opt.default is not None:
         extras.append(
-            ':default: %s'
+            ':default: ``%s``'
             % (
-                ', '.join(str(d) for d in opt.default)
+                ', '.join(repr(d) for d in opt.default)
                 if isinstance(opt.default, (list, tuple))
-                else opt.default,
+                else repr(opt.default),
             )
         )
 
@@ -121,6 +149,7 @@ def _format_help(help_string: str) -> ty.Generator[str, None, None]:
     yield ''
 
 
+@_process_lines("sphinx-click-process-description")
 def _format_description(ctx: click.Context) -> ty.Generator[str, None, None]:
     """Format the description for a given `click.Command`.
 
@@ -132,6 +161,7 @@ def _format_description(ctx: click.Context) -> ty.Generator[str, None, None]:
         yield from _format_help(help_string)
 
 
+@_process_lines("sphinx-click-process-usage")
 def _format_usage(ctx: click.Context) -> ty.Generator[str, None, None]:
     """Format the usage for a `click.Command`."""
     yield '.. code-block:: shell'
@@ -141,30 +171,40 @@ def _format_usage(ctx: click.Context) -> ty.Generator[str, None, None]:
     yield ''
 
 
-def _format_option(opt: click.Option) -> ty.Generator[str, None, None]:
-    """Format the output for a `click.Option`."""
-    opt_help = _get_help_record(opt)
+def _format_option(
+    ctx: click.Context, opt: click.core.Option
+) -> ty.Generator[str, None, None]:
+    """Format the output for a `click.core.Option`."""
+    opt_help = _get_help_record(ctx, opt)
 
     yield '.. option:: {}'.format(opt_help[0])
     if opt_help[1]:
         yield ''
+        bar_enabled = False
         for line in statemachine.string2lines(
             ANSI_ESC_SEQ_RE.sub('', opt_help[1]), tab_width=4, convert_whitespace=True
         ):
+            if line == '\b':
+                bar_enabled = True
+                continue
+            if line == '':
+                bar_enabled = False
+            line = '| ' + line if bar_enabled else line
             yield _indent(line)
 
 
+@_process_lines("sphinx-click-process-options")
 def _format_options(ctx: click.Context) -> ty.Generator[str, None, None]:
     """Format all `click.Option` for a `click.Command`."""
     # the hidden attribute is part of click 7.x only hence use of getattr
     params = [
         param
         for param in ctx.command.params
-        if isinstance(param, click.Option) and not getattr(param, 'hidden', False)
+        if isinstance(param, click.core.Option) and not getattr(param, 'hidden', False)
     ]
 
     for param in params:
-        for line in _format_option(param):
+        for line in _format_option(ctx, param):
             yield line
         yield ''
 
@@ -178,8 +218,16 @@ def _format_argument(arg: click.Argument) -> ty.Generator[str, None, None]:
             'Required' if arg.required else 'Optional', '(s)' if arg.nargs != 1 else ''
         )
     )
+    # Subclasses of click.Argument may add a `help` attribute (like typer.main.TyperArgument)
+    help = getattr(arg, 'help', None)
+    if help:
+        yield ''
+        help_string = ANSI_ESC_SEQ_RE.sub('', help)
+        for line in _format_help(help_string):
+            yield _indent(line)
 
 
+@_process_lines("sphinx-click-process-arguments")
 def _format_arguments(ctx: click.Context) -> ty.Generator[str, None, None]:
     """Format all `click.Argument` for a `click.Command`."""
     params = [x for x in ctx.command.params if isinstance(x, click.Argument)]
@@ -191,7 +239,7 @@ def _format_arguments(ctx: click.Context) -> ty.Generator[str, None, None]:
 
 
 def _format_envvar(
-    param: ty.Union[click.Option, click.Argument]
+    param: ty.Union[click.core.Option, click.Argument]
 ) -> ty.Generator[str, None, None]:
     """Format the envvars of a `click.Option` or `click.Argument`."""
     yield '.. envvar:: {}'.format(param.envvar)
@@ -207,9 +255,19 @@ def _format_envvar(
     yield _indent('Provide a default for :option:`{}`'.format(param_ref))
 
 
+@_process_lines("sphinx-click-process-envars")
 def _format_envvars(ctx: click.Context) -> ty.Generator[str, None, None]:
     """Format all envvars for a `click.Command`."""
-    params = [x for x in ctx.command.params if x.envvar]
+
+    auto_envvar_prefix = ctx.auto_envvar_prefix
+    if auto_envvar_prefix is not None:
+        params = []
+        for param in ctx.command.params:
+            if not param.envvar:
+                param.envvar = f"{auto_envvar_prefix}_{param.name.upper()}"
+            params.append(param)
+    else:
+        params = [x for x in ctx.command.params if x.envvar]
 
     for param in params:
         yield '.. _{command_name}-{param_name}-{envvar}:'.format(
@@ -237,6 +295,7 @@ def _format_subcommand(command: click.Command) -> ty.Generator[str, None, None]:
             yield _indent(line)
 
 
+@_process_lines("sphinx-click-process-epilog")
 def _format_epilog(ctx: click.Context) -> ty.Generator[str, None, None]:
     """Format the epilog for a given `click.Command`.
 
@@ -250,7 +309,7 @@ def _format_epilog(ctx: click.Context) -> ty.Generator[str, None, None]:
 def _get_lazyload_commands(ctx: click.Context) -> ty.Dict[str, click.Command]:
     commands = {}
     for command in ctx.command.list_commands(ctx):
-        commands[command] = ctx.command.get_command(ctx.command, command)
+        commands[command] = ctx.command.get_command(ctx, command)
 
     return commands
 
@@ -272,7 +331,7 @@ def _filter_commands(
 
 def _format_command(
     ctx: click.Context,
-    nested: str,
+    nested: NestedT,
     commands: ty.Optional[ty.List[str]] = None,
 ) -> ty.Generator[str, None, None]:
     """Format the output of `click.Command`."""
@@ -348,7 +407,7 @@ def _format_command(
         yield ''
 
 
-def nested(argument: ty.Optional[str]) -> ty.Optional[str]:
+def nested(argument: ty.Optional[str]) -> NestedT:
     values = (NESTED_FULL, NESTED_SHORT, NESTED_NONE, None)
 
     if argument not in values:
@@ -357,11 +416,10 @@ def nested(argument: ty.Optional[str]) -> ty.Optional[str]:
             % directives.format_values(values)
         )
 
-    return argument
+    return ty.cast(NestedT, argument)
 
 
 class ClickDirective(rst.Directive):
-
     has_content = False
     required_arguments = 1
     option_spec = {
@@ -382,7 +440,8 @@ class ClickDirective(rst.Directive):
             )
 
         try:
-            mod = __import__(module_name, globals(), locals(), [attr_name])
+            with mock(self.env.config.sphinx_click_mock_imports):
+                mod = __import__(module_name, globals(), locals(), [attr_name])
         except (Exception, SystemExit) as exc:  # noqa
             err_msg = 'Failed to import "{}" from "{}". '.format(attr_name, module_name)
             if isinstance(exc, SystemExit):
@@ -413,10 +472,10 @@ class ClickDirective(rst.Directive):
         name: str,
         command: click.Command,
         parent: ty.Optional[click.Context],
-        nested: str,
+        nested: NestedT,
         commands: ty.Optional[ty.List[str]] = None,
         semantic_group: bool = False,
-    ) -> ty.List[section]:
+    ) -> ty.List[nodes.section]:
         """Generate the relevant Sphinx nodes.
 
         Format a `click.Group` or `click.Command`.
@@ -447,8 +506,9 @@ class ClickDirective(rst.Directive):
 
         # Summary
         source_name = ctx.command_path
-        result = statemachine.ViewList()
+        result = statemachine.StringList()
 
+        ctx.meta["sphinx-click-env"] = self.env
         if semantic_group:
             lines = _format_description(ctx)
         else:
@@ -486,7 +546,7 @@ class ClickDirective(rst.Directive):
 
         return [section]
 
-    def run(self) -> ty.Iterable[section]:
+    def run(self) -> ty.Sequence[nodes.section]:
         self.env = self.state.document.settings.env
 
         command = self._load_module(self.arguments[0])
@@ -494,7 +554,7 @@ class ClickDirective(rst.Directive):
         if 'prog' not in self.options:
             raise self.error(':prog: must be specified')
 
-        prog_name = self.options.get('prog')
+        prog_name = self.options['prog']
         show_nested = 'show-nested' in self.options
         nested = self.options.get('nested')
 
@@ -510,16 +570,30 @@ class ClickDirective(rst.Directive):
                 )
                 nested = NESTED_FULL if show_nested else NESTED_SHORT
 
-        commands = [
-            command.strip() for command in self.options.get('commands', '').split(',')
-        ]
+        commands = None
+        if self.options.get('commands'):
+            commands = [
+                command.strip() for command in self.options['commands'].split(',')
+            ]
 
         return self._generate_nodes(prog_name, command, None, nested, commands)
 
 
 def setup(app: application.Sphinx) -> ty.Dict[str, ty.Any]:
+    # Need autodoc to support mocking modules
+    app.setup_extension('sphinx.ext.autodoc')
     app.add_directive('click', ClickDirective)
 
+    app.add_event("sphinx-click-process-description")
+    app.add_event("sphinx-click-process-usage")
+    app.add_event("sphinx-click-process-options")
+    app.add_event("sphinx-click-process-arguments")
+    app.add_event("sphinx-click-process-envvars")
+    app.add_event("sphinx-click-process-epilog")
+    app.add_config_value(
+        'sphinx_click_mock_imports', lambda config: config.autodoc_mock_imports, 'env'
+    )
+
     return {
         'parallel_read_safe': True,
         'parallel_write_safe': True,
