@@ -170,6 +170,62 @@ class CommandTestCase(unittest.TestCase):
             '\n'.join(output),
         )
 
+    def test_help_argument(self):
+        """Validate a help text for arguments.
+
+        While click only provides the help attribute for options, but not for arguments,
+        it allows customization with subclasses.
+        """
+
+        class CustomArgument(click.Argument):
+            def __init__(self, *args, help=None, **kwargs):
+                super().__init__(*args, **kwargs)
+                self.help = help
+
+        @click.command()
+        @click.option('--option', help='A sample option')
+        @click.argument('ARG', help='A sample argument', cls=CustomArgument)
+        @click.argument('ARG_NO_HELP', cls=CustomArgument)
+        def foobar(bar):
+            """A sample command."""
+            pass
+
+        ctx = click.Context(foobar, info_name='foobar')
+        output = list(ext._format_command(ctx, nested='short'))
+
+        self.assertEqual(
+            textwrap.dedent(
+                """
+        A sample command.
+
+        .. program:: foobar
+        .. code-block:: shell
+
+            foobar [OPTIONS] ARG ARG_NO_HELP
+
+        .. rubric:: Options
+
+        .. option:: --option <option>
+
+            A sample option
+
+        .. rubric:: Arguments
+
+        .. option:: ARG
+
+            Required argument
+
+            A sample argument
+
+
+        .. option:: ARG_NO_HELP
+
+            Required argument
+        """
+            ).lstrip(),
+            '\n'.join(output),
+        )
+
     def test_defaults(self):
         """Validate formatting of user documented defaults."""
 
@@ -192,6 +248,8 @@ class CommandTestCase(unittest.TestCase):
             '--only-show-default',
             show_default="Some default computed at runtime!",
         )
+        @click.option('--string-default', default="abc", show_default=True)
+        @click.option('--empty-string-default', default="", show_default=True)
         def foobar(bar):
             """A sample command."""
             pass
@@ -217,7 +275,7 @@ class CommandTestCase(unittest.TestCase):
 
         .. option:: --param <param>
 
-            :default: ``Something computed at runtime``
+            :default: ``'Something computed at runtime'``
 
         .. option:: --group <group>
 
@@ -225,7 +283,49 @@ class CommandTestCase(unittest.TestCase):
 
         .. option:: --only-show-default <only_show_default>
 
-            :default: ``Some default computed at runtime!``
+            :default: ``'Some default computed at runtime!'``
+
+        .. option:: --string-default <string_default>
+
+            :default: ``'abc'``
+
+        .. option:: --empty-string-default <empty_string_default>
+
+            :default: ``''``
+        """
+            ).lstrip(),
+            '\n'.join(output),
+        )
+
+    def test_show_default(self):
+        """Validate formatting of show_default via context_settings."""
+
+        @click.command(context_settings={"show_default": True})
+        @click.option('--no-set', default=0)
+        @click.option('--set-false', default=0, show_default=False)
+        def foobar():
+            """A sample command."""
+            pass
+
+        ctx = click.Context(foobar, info_name='foobar', show_default=True)
+        output = list(ext._format_command(ctx, nested='short'))
+        self.assertEqual(
+            textwrap.dedent(
+                """
+        A sample command.
+
+        .. program:: foobar
+        .. code-block:: shell
+
+            foobar [OPTIONS]
+
+        .. rubric:: Options
+
+        .. option:: --no-set <no_set>
+
+            :default: ``0``
+
+        .. option:: --set-false <set_false>
         """
             ).lstrip(),
             '\n'.join(output),
@@ -348,7 +448,7 @@ class CommandTestCase(unittest.TestCase):
 
         .. option:: --param <param>
 
-            :default: ``Something computed at runtime``
+            :default: ``'Something computed at runtime'``
 
         A sample epilog.
         """
@@ -500,6 +600,8 @@ that will be rewrapped again.
 class GroupTestCase(unittest.TestCase):
     """Validate basic ``click.Group`` instances."""
 
+    maxDiff = None
+
     def test_no_parameters(self):
         """Validate a `click.Group` with no parameters.
 
@@ -592,6 +694,8 @@ class GroupTestCase(unittest.TestCase):
 class NestedCommandsTestCase(unittest.TestCase):
     """Validate ``click.Command`` instances inside ``click.Group`` instances."""
 
+    maxDiff = None
+
     @staticmethod
     def _get_ctx():
         @click.group()
@@ -686,6 +790,8 @@ class NestedCommandsTestCase(unittest.TestCase):
 class CommandFilterTestCase(unittest.TestCase):
     """Validate filtering of commands."""
 
+    maxDiff = None
+
     @staticmethod
     def _get_ctx():
         @click.group()
@@ -758,6 +864,8 @@ class CommandFilterTestCase(unittest.TestCase):
 class CustomMultiCommandTestCase(unittest.TestCase):
     """Validate ``click.MultiCommand`` instances."""
 
+    maxDiff = None
+
     def test_basics(self):
         """Validate a custom ``click.MultiCommand`` with no parameters.
 
@@ -874,6 +982,8 @@ class CustomMultiCommandTestCase(unittest.TestCase):
 class CommandCollectionTestCase(unittest.TestCase):
     """Validate ``click.CommandCollection`` instances."""
 
+    maxDiff = None
+
     def test_basics(self):
         "Validate a ``click.CommandCollection`` with grouped outputs."
 
@@ -945,6 +1055,8 @@ class CommandCollectionTestCase(unittest.TestCase):
 class AutoEnvvarPrefixTestCase(unittest.TestCase):
     """Validate ``click auto_envvar_prefix``-setup instances."""
 
+    maxDiff = None
+
     def test_basics(self):
         """Validate a click application with ``auto_envvar_prefix`` option enabled."""
 
